

package hu.bme.mit.cps.k8slab.brightnesspublisher;


/* SolarPerformancePublisher.java

A publication of data of type SolarPerformance

This file is derived from code automatically generated by the rtiddsgen 
command:

rtiddsgen -language java -example <arch> .idl

Example publication of type SolarPerformance automatically generated by 
'rtiddsgen' To test them follow these steps:

(1) Compile this file and the example subscription.

(2) Start the subscription on the same domain used for RTI Data Distribution
Service with the command
java SolarPerformanceSubscriber <domain_id> <sample_count>

(3) Start the publication on the same domain used for RTI Data Distribution
Service with the command
java SolarPerformancePublisher <domain_id> <sample_count>

(4) [Optional] Specify the list of discovery initial peers and 
multicast receive addresses via an environment variable or a file 
(in the current working directory) called NDDS_DISCOVERY_PEERS.  

You can run any number of publishers and subscribers programs, and can 
add and remove them dynamically from the domain.

Example:

To run the example application on domain <domain_id>:

Ensure that $(NDDSHOME)/lib/<arch> is on the dynamic library path for
Java.                       

On Unix: 
add $(NDDSHOME)/lib/<arch> to the 'LD_LIBRARY_PATH' environment
variable

On Windows:
add %NDDSHOME%\lib\<arch> to the 'Path' environment variable

Run the Java applications:

java -Djava.ext.dirs=$NDDSHOME/lib/java SolarPerformancePublisher <domain_id>

java -Djava.ext.dirs=$NDDSHOME/lib/java SolarPerformanceSubscriber <domain_id>        
*/

import java.io.IOException;
import java.io.Reader;
import java.math.MathContext;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;

import com.opencsv.bean.CsvToBean;
import com.opencsv.bean.CsvToBeanBuilder;
import com.rti.dds.domain.*;
import com.rti.dds.infrastructure.*;
import com.rti.dds.publication.*;
import com.rti.dds.topic.*;
import com.rti.ndds.config.*;

import hu.bme.mit.cps.k8slab.Brightness;
import hu.bme.mit.cps.k8slab.BrightnessDataWriter;
import hu.bme.mit.cps.k8slab.BrightnessTypeSupport;
import hu.bme.mit.cps.k8slab.CSVBrightness;

// ===========================================================================

public class BrightnessPublisher {
    // -----------------------------------------------------------------------
    // Public Methods
    // -----------------------------------------------------------------------

    public static void main(String[] args) {
        
    	int sensorID = 0;
        if (args.length >= 1) {
            sensorID = Integer.valueOf(args[0]).intValue();
        }
    	
    	int sensor1delay = 0;
        if (args.length >= 2) {
            sensor1delay = Integer.valueOf(args[1]).intValue();
        }
        
    	int sensor2delay = 0;
        if (args.length >= 3) {
            sensor2delay = Integer.valueOf(args[2]).intValue();
        }
    	
    	// --- Get domain ID --- //
        int domainId = 0;
        if (args.length >= 4) {
            domainId = Integer.valueOf(args[3]).intValue();
        }

        // -- Get max loop count; 0 means infinite loop --- //
        int sampleCount = 0;
        if (args.length >= 5) {
            sampleCount = Integer.valueOf(args[4]).intValue();
        }

        /* Uncomment this to turn on additional logging
        Logger.get_instance().set_verbosity_by_category(
            LogCategory.NDDS_CONFIG_LOG_CATEGORY_API,
            LogVerbosity.NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
        */

        // --- Run --- //
        publisherMain(sensorID, sensor1delay, sensor2delay, domainId, sampleCount);
    }

    // -----------------------------------------------------------------------
    // Private Methods
    // -----------------------------------------------------------------------

    // --- Constructors: -----------------------------------------------------

    private BrightnessPublisher() {
        super();
    }

    // -----------------------------------------------------------------------

    private static void publisherMain(int sensorID, int sensor1delay, int sensor2delay, int domainId, int sampleCount) {

        DomainParticipant participant = null;
        Publisher publisher = null;
        Topic topic = null;
        BrightnessDataWriter writer = null;

        List<CSVBrightness> csvData = null;
        
        // --- Reading data --- //
        
        try {
			Reader reader = Files.newBufferedReader(Paths.get("./lightsensordata.csv"));
			
			CsvToBean<CSVBrightness> csvToBean = new CsvToBeanBuilder(reader)
                    .withType(CSVBrightness.class)
                    .withIgnoreLeadingWhiteSpace(true)
                    .build();
			
			 csvData = csvToBean.parse();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
        
        try {
            // --- Create participant --- //

            /* To customize participant QoS, use
            the configuration file
            USER_QOS_PROFILES.xml */

            participant = DomainParticipantFactory.TheParticipantFactory.
            create_participant(
                domainId, DomainParticipantFactory.PARTICIPANT_QOS_DEFAULT,
                null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (participant == null) {
                System.err.println("create_participant error\n");
                return;
            }        

            // --- Create publisher --- //

            /* To customize publisher QoS, use
            the configuration file USER_QOS_PROFILES.xml */

            publisher = participant.create_publisher(
                DomainParticipant.PUBLISHER_QOS_DEFAULT, null /* listener */,
                StatusKind.STATUS_MASK_NONE);
            if (publisher == null) {
                System.err.println("create_publisher error\n");
                return;
            }                   

            // --- Create topic --- //

            /* Register type before creating topic */
            String typeName = BrightnessTypeSupport.get_type_name();
            BrightnessTypeSupport.register_type(participant, typeName);

            /* To customize topic QoS, use
            the configuration file USER_QOS_PROFILES.xml */

            topic = participant.create_topic(
                "BrightnessTopic",
                typeName, DomainParticipant.TOPIC_QOS_DEFAULT,
                null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (topic == null) {
                System.err.println("create_topic error\n");
                return;
            }           

            // --- Create writer --- //

            /* To customize data writer QoS, use
            the configuration file USER_QOS_PROFILES.xml */

            writer = (BrightnessDataWriter)
            publisher.create_datawriter(
                topic, Publisher.DATAWRITER_QOS_DEFAULT,
                null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (writer == null) {
                System.err.println("create_datawriter error\n");
                return;
            }           

            // --- Write --- //

            /* Create data sample for writing */
            Brightness instance = new Brightness();
            instance.SensorID = sensorID;
            
            
            InstanceHandle_t instance_handle = InstanceHandle_t.HANDLE_NIL;
            /* For a data type that has a key, if the same instance is going to be
            written multiple times, initialize the key here
            and register the keyed instance prior to writing */
            //instance_handle = writer.register_instance(instance);

            final long sendPeriodMillis = 10 * 1000; // 4 seconds
            
            int i = 10-sensor1delay;
            int j = 10-sensor2delay;
            
            if(csvData != null) {
	            for (int count = 0;
	            (sampleCount == 0) || (count < sampleCount);
	            ++count) {
	                
	
	                /* Modify the instance to be written here */
	                
	                instance.Value = (csvData.get(i).Light + csvData.get(j).Light)/2;
	                
	                System.out.println("Writing: " + instance.Value + "\n\tfrom: " + csvData.get(i).Light + ", " + csvData.get(j).Light);
	                
	                i++;
	                if(i == csvData.size()) {
	                	i = 0;
	                }
	                
	                j++;
	                if(j == csvData.size()) {
	                	j = 0;
	                }
	                
	                /* Write data */
	                writer.write(instance, instance_handle);
	                try {
	                    Thread.sleep(sendPeriodMillis);
	                } catch (InterruptedException ix) {
	                    System.err.println("INTERRUPTED");
	                    break;
	                }
	            }
            } else {
            	System.err.println("CSV Data error!");
            	return;
            }

            //writer.unregister_instance(instance, instance_handle);

        } finally {

            // --- Shutdown --- //

            if(participant != null) {
                participant.delete_contained_entities();

                DomainParticipantFactory.TheParticipantFactory.
                delete_participant(participant);
            }
            /* RTI Data Distribution Service provides finalize_instance()
            method for people who want to release memory used by the
            participant factory singleton. Uncomment the following block of
            code for clean destruction of the participant factory
            singleton. */
            //DomainParticipantFactory.finalize_instance();
        }
    }
}

